
//-BBMobile little library
//-MIT license Copyright (c) 2020 Projekt BBMagic.net
//----------------------------------------------------------------------------------
//-Ta maa biblioteka umozliwia tworzenie mobilnych aplikacji przy uyciu modulu BBMobile
//-moesz go bezpatnie uywa, kopiowa, modyfikowa, czy i redystrybuowa prywatnie
//-oraz komercyjnnie zawierajc t notatk o prawach autorskich (licencja MIT)
//-wicej info www.bbmagic.net
//----------------------------------------------------------------------------------
//-This little library is suitable for building mobile applications using BBMobile module
//-you can use, copy, modify, merge, publish, distribute, and/or sell copies
//-free of charge uder MIT license
//-more info www.bbmagic.net
//----------------------------------------------------------------------------------

	//-checks if Bluetooth LE connection is opened
	inline uint8_t bbmobile_check_ble_conn(void) ;
	//-checks if there is new data from BMobile
	inline uint8_t bbmobile_is_data(void) ;
	//-sends json string from flash memory to BBMobile App
	uint8_t bbmobile_send_json(const char *j) ;
	//-waits ms - defined in bbmobile_uartXX.c
	void bbmobile_wait_ms(uint16_t ms) ;
	//-sends unsigned uint16_t number in decimal format to BBMobile UART
	void bbmobile_send_dec_2app(uint16_t u16number, uint8_t w) ;
	//-sends signed int16_t number in +/-decimal format to BBMobile UART
	void bbmobile_send_sigdec_2app(int16_t s16number, uint8_t w) ;
	//-sends byte in hex format to BBMobile UART
	void bbmobile_send_hex_2app(uint8_t b) ;
	//-sends data from flash to BBMobile and doesnt wait for ack
#define bbmobile_send_2app(flash_string)		(bbmobile_uart_send_flash((const char *)PSTR(flash_string)))
	//-sends data from flash to BBMobile app and waits for ack: $ok
#define bbmobile_send_2app_ack(flash_string)	(bbmobile_send_json((const char *)PSTR(flash_string)))
	//-sends data from flash to BBMobile module and waits for ack: >OK
#define bbmobile_send_2chip_ack(flash_string)	(_bbmobile_send_2chip_ack((const char *)PSTR(flash_string)))
	//-compares cnt number of bytes
#define cmp_flash_ram_bytes(flash, ram, cnt)	(_cmp_flash_ram_bytes((const char *)PSTR(flash), (uint8_t *)ram, (uint8_t)cnt))


#define MSG_MODULE	0
#define MSG_APP		1

	static uint8_t _cmp_flash_ram_bytes(const char *flash, uint8_t *ram, uint8_t cnt) ;
	
	volatile uint8_t ble_conn ;		//0-no BLE connection; 1-BLE connection is open

#include "bbmobile_uart04.c"

//===========================================================================================
//-bbmobile_send_dec_2app - sends uint16_t number in decimal format
//-------gets
// u16number - unsigned number to send
// w - number width
//-------returns
//===========================================================================================
void bbmobile_send_dec_2app(uint16_t u16number, uint8_t w)
{
    uint8_t i, f = 0 ;
    uint16_t tmp ;

    i = u16number/10000 ;
    if(i > 0)
    {
        bbmobile_uart_send_byte(i+48) ;
        tmp = (uint16_t)(i) * 10000 ;
        u16number -=tmp ;
        f++ ;
    }else if(w > 4) bbmobile_uart_send_byte(48) ;

    i = u16number/1000 ;
    if(i > 0) f++ ;
    if(f != 0) bbmobile_uart_send_byte(i+48) ;
    else if(w > 3) bbmobile_uart_send_byte(48) ;
    tmp = (uint16_t)(i) * 1000 ;
    u16number -=tmp ;

    i = u16number/100 ;
    if(i > 0) f++ ;
    if(f != 0) bbmobile_uart_send_byte(i+48) ;
    else if(w > 2) bbmobile_uart_send_byte(48) ;
    tmp = (uint16_t)(i) * 100 ;
    u16number -=tmp ;

    i = u16number/10 ;
    if(i > 0) f++ ;
    if(f != 0) bbmobile_uart_send_byte(i+48) ;
    else if(w > 1) bbmobile_uart_send_byte(48) ;
    tmp = (uint16_t)(i) * 10 ;
    u16number -=tmp ;

    i = (uint8_t)(u16number) ;
    bbmobile_uart_send_byte(i+48) ;
}

//===========================================================================================
//-bbmobile_send_dec_2app - sends s16 number in +/-decimal format
//-------gets
// s16number - signed number to send
// w - number width
//-------returns
//===========================================================================================
void bbmobile_send_sigdec_2app(int16_t s16number, uint8_t w)
{
	if(s16number < 0)
	{
	 	bbmobile_uart_send_byte('-') ;
    	//kod U2 -> [-2exp(n-1) , 2exp(n-1)-1] - > dla 10bitow [-512 , 511]
    	//-Aby zamieni liczb w U2 na przeciwn, naley: 1.dokona inwersji bitw, 2. zwikszy wynik o 1.
        s16number = ~s16number ;
        s16number++ ;
	}
	bbmobile_send_dec_2app((uint16_t)s16number,w) ;
}

//==========================================================================
//-bbmobile_send_nibble
//-------gets
//-------returns
//==========================================================================
void bbmobile_send_nibble(uint8_t b)
{
	b&= 0x0f ;
	if(b > 9) b+=( 'A'-10 ) ;
	else b+='0';
	bbmobile_uart_send_byte(b) ;
}

//===========================================================================================
//-bbmobile_send_hex_2app - sends byte in hex format to BBMobile uart
//-------gets
// byte to send
//-------returns
//===========================================================================================
void bbmobile_send_hex_2app(uint8_t b)
{
	bbmobile_send_nibble(b>>4) ;
	bbmobile_send_nibble(b) ;
}

//==========================================================================
//-bbmobile_check_ble_conn - checks if Bluetooth LE connection is opened
//-------gets
//-------returns
//-0 - no BLE connection
//-1 - BLE connection established
//==========================================================================
inline uint8_t bbmobile_check_ble_conn(void)
{
	return(ble_conn) ;
}

//===========================================================================================
//-bbmobile_rx_enable
//-------gets
//-------returns
//===========================================================================================
inline void bbmobile_rx_enable(void)
{
	bbm_msg_rdy =0 ;	//-clear new msg flag
}

//==========================================================================
//-bbmobile_is_data - checks if there is new data from BBMobile
//-------gets
//-------returns
//-0 - no data
//- !=0 - there is new data from BBMobile App
//==========================================================================
inline uint8_t bbmobile_is_data(void)
{
	return(bbm_msg_rdy) ;
}

//==========================================================================
//-bbmobile_wait_msg - waits form UART message for timeout_ms miliseconds
//-------gets
// timeout_ms - miliseconds to wait
// type - 0 - non connection mode msg, 1-BLE connection mode msg
//-------returns
// 0-message received
// 1-timeout
// 2-BLE connection closed
//==========================================================================
static uint8_t bbmobile_wait_msg(uint16_t timeout_ms, uint8_t type)
{
	timeout_ms >>=3 ;
	do
	{
		bbmobile_wait_ms(8) ;
		if( bbmobile_is_data() )
			return(0) ;
		if(type == MSG_APP)
			if(ble_conn == 0)
				return(2) ;
	}while(timeout_ms-- >0) ;
	return(1) ;
}

//==========================================================================
//-bbmobile_send_json - sends json string from flash memory to BBMobile App
//-------gets
//-pointer to JSON string in flash memory
//-------returns
// 0 - sent and get $ok
// 2 - JSON is incorrect - get $nok
// 4 - no answer - timeout
// 6 - ble connection not open
//==========================================================================
uint8_t bbmobile_send_json(const char *j)
{
	if( ble_conn ==0)
		return(6) ;

	bbmobile_rx_enable() ;
	bbmobile_uart_send_flash(j) ;
	if(bbmobile_wait_msg(2000, MSG_APP) ==0)
	{
		if( cmp_flash_ram_bytes("$ok", bbm_buf, 3) )  //-JSON correct
		{
			bbmobile_rx_enable() ;
			return(0) ;
		}
		if( cmp_flash_ram_bytes("$nok", bbm_buf, 4) )  //-JSON incorrect
		{
			bbmobile_rx_enable() ;
			return(2) ;
		}
	}
	return(4) ;
}

//==========================================================================
//-_bbmobile_send_2chip_ack - sends string data from flash to BBMobile module chip
//-------gets
//-pointer to string in flash
//-------returns
// 0 - sent and get ack - get > ack
// 2 - command is incorrect - get >ERR
// 4 - no answer - timeout
// 6 - ble connection is opened - communication with chip is impossible
//==========================================================================
static uint8_t _bbmobile_send_2chip_ack(const char *cmd)
{
	if( ble_conn ==1) return(6) ;

	bbmobile_rx_enable() ;
	bbmobile_uart_send_flash(cmd) ;
	if(bbmobile_wait_msg(500, MSG_MODULE) ==0 )
	{
		if( cmp_flash_ram_bytes(">ERR", bbm_buf, 4) )  //-error answer
		{
			bbmobile_rx_enable() ;
			return(2) ;
		}else
		{
			bbmobile_rx_enable() ;
			return(0) ;
		}
	}
	return(4) ;
}

//===========================================================================================
//- _cmp_flash_ram_bytes - compares RAM to flash bytes
//-------gets
//- flash - pointer to bytes in flash
//- ram - pointer to bytes in ram
//- cnt - number of bytes to compare
//-------returns
// 0 - dont match
// 1 - bytes match
//===========================================================================================
static uint8_t _cmp_flash_ram_bytes(const char *flash, uint8_t *ram, uint8_t cnt)
{
    uint8_t b, i ;

    for(i=0; i<cnt; i++)
    {
		b = pgm_read_byte(flash) ;
        if(b != ram[i]) return 0 ;   //-dont match
        flash++ ;
    }
    return 1 ;                      //-match
}

