
//-BBMobile little library
//-MIT license Copyright (c) 2020 Projekt BBMagic.net
//----------------------------------------------------------------------------------
//-Ta maa biblioteka umozliwia tworzenie mobilnych aplikacji przy uyciu modulu BBMobile
//-moesz go bezpatnie uywa, kopiowa, modyfikowa, czy i redystrybuowa prywatnie
//-oraz komercyjnnie zawierajc t notatk o prawach autorskich (licencja MIT)
//-wicej info www.bbmagic.net
//----------------------------------------------------------------------------------
//-This little library is suitable for building mobile applications using BBMobile module
//-you can use, copy, modify, merge, publish, distribute, and/or sell copies
//-free of charge uder MIT license
//-more info www.bbmagic.net
//----------------------------------------------------------------------------------

//-define your CPU clock for _delay_ms function
#define F_CPU	1000000UL
//#define F_CPU	8000000UL
//-define UART data reg name
#define UART_DATA_REG	UDR0
//-you can change ram buffer size - it must contain the biggest string from BBMobile app
#define BBM_BUF_SIZE	64	//-max 255

	uint8_t bbm_buf[BBM_BUF_SIZE] ;		//-user buffer for data from BBMobile
	uint8_t bbm_i ;						//-buffer index
	uint8_t uart_buf[BBM_BUF_SIZE] ;	//-uart lib buffer for data from BBMobile
	uint8_t uart_i ;                    //-buffer index

#define bbmobile_send_str_flash(string) (bbmobile_uart_send_flash((const char *)PSTR(string)))

	volatile uint8_t bbm_msg_rdy ;	//-0-no new msg from BBMobile in bbm_buf ; 1-new msg from BBMobile in bbm_buf

#include <util/delay.h>

//===========================================================================================
//-bbmobile_wait_ms - waits time in milisecond
//-------gets
//-------returns
//===========================================================================================
void bbmobile_wait_ms(uint16_t ms)
{
	_delay_ms(ms) ;
}

//==========================================================================
//-bbmobile_uart_init - change reg/bits names for your chip
//-------gets
//-ubrr_reg - baudrate reg value
//-------returns
//==========================================================================
void bbmobile_uart_init(uint16_t ubrr_reg)
{
	//-Set baudrate
	UBRR0H = (uint8_t)(ubrr_reg>>8) ;
	UBRR0L = (uint8_t)ubrr_reg ;
	UCSR0A |= (1<<U2X0) ;		//-Double UART speed
	//-Enable transmitter, receiver and rec interrupt
	UCSR0B = (1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0) ;
	UCSR0C = (3<<UCSZ00) ;		//-Set frame format: Asyncronous 8N1
}

//==========================================================================
//-bbmobile_uart_tx_wait_free - change reg/bits names for your chip
//-------gets
//-------returns
//==========================================================================
inline void bbmobile_uart_tx_wait_free(void)
{
	//-wait UDRE is empty - tx buffer is ready to receive new data
	while(!(UCSR0A & (1<<UDRE0))) ;
}


//-------------------------------------------------------------------------
//-YOU DO NOT NEED TO CHANGE ANYTHING BELOW !!
//-------------------------------------------------------------------------

//==========================================================================
//-bbmobile_uart_send_byte
//-------gets
//-byte to send
//-------returns
//==========================================================================
inline void bbmobile_uart_send_byte(uint8_t b)
{
	bbmobile_uart_tx_wait_free() ;		//-zaczekaj na zakonaczenie poprzedniej
	UART_DATA_REG = b ; 				//-wyslij znak
}

//==========================================================================
//-bbmobile_uart_send_flash
//-------gets
// pointer to string in flash memory
//-------returns
//==========================================================================
void bbmobile_uart_send_flash(const char *data)
{
	uint8_t bb ;
	while(1)
	{
		bb = pgm_read_byte(data) ;
		bbmobile_uart_tx_wait_free() ;		//-zaczekaj na zakonaczenie poprzedniej
		if(bb == 0) return ;
		UART_DATA_REG = bb ;
		data++ ;
	} ;
}

//==========================================================================
//-bbmobile_uart_send_ram
//-------gets
// char *data - pointer to data in RAM
// uint8_t cnt - number of bytes to send
//-------returns
//==========================================================================
void bbmobile_uart_send_ram(char *data, uint8_t cnt)
{
	while(cnt-- >0)
	{
		bbmobile_uart_tx_wait_free() ;		//-zaczekaj na zakonaczenie poprzedniej
		UART_DATA_REG =*data ;
		data++ ;
	} ;
}

//==========================================================================
//-BBobile UART receive byte interrupt handler
//-------gets
//-------returns
//==========================================================================
SIGNAL(USART_RX_vect)
{
	uint8_t bb ;

	if(uart_i >= BBM_BUF_SIZE)		//-buffer overflow
		uart_i =0 ;					//-clear buffer - wait for new message

	bb = UART_DATA_REG ;
	if(uart_i > 0)					//-frame is receiving now
	{
		uart_buf[uart_i] =bb ;		//-add new byte to buffer
		uart_i++ ;
		if(bb == '\r')										//-end of frame
		{
			if( cmp_flash_ram_bytes(">CON", uart_buf, 4) )   //-connection opened
			{
				ble_conn =1 ;
			}else if( cmp_flash_ram_bytes(">DCON", uart_buf, 5) )  //-connection closed
			{
				ble_conn =0 ;
			}else
			{
				if(bbm_msg_rdy ==0)						//-previous message processed
				{
					bbm_i =0 ;
					while(uart_i-- >0)					//-copy new message to bbm_buf
					{
						bbm_buf[bbm_i] = uart_buf[bbm_i] ;
						bbm_i++ ;
					} ;
					bbm_msg_rdy =1 ;					//-set new msg flag
				}
			}
			uart_i =0 ;									//-clear buffer - wait for new message
		}
	}else												//-waiting for new frame
	{
		if((bb == '>') || (bb == '$'))
		{
			uart_i =0 ;
			uart_buf[uart_i] =bb ;						//-add new byte to buffer
			uart_i++ ;
		}
	}
}

