

//----------------------------------------------------------------------------------
// BBMobile COUNTER
//-MIT license Copyright (c) 2020 Projekt BBMagic.net
//----------------------------------------------------------------------------------
//-to jest demonstracyjny program dla atmega88 zliczajcy impulsy na wejciu INT0
//-moesz go bezpatnie uywa, kopiowa, modyfikowa, czy i redystrybuowa prywatnie
//-oraz komercyjnnie zawierajc t notatk o prawach autorskich (licencja MIT)
//-wicej info www.bbmagic.net
//----------------------------------------------------------------------------------
//-this is demo code for atmega88 chip that counts LO level pulses on INT0 input
//-you can use, copy, modify, merge, publish, distribute, and/or sell copies
//-free of charge uder MIT license
//-more info www.bbmagic.net
//----------------------------------------------------------------------------------

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "bbmobile_lib04.c"

#define LED_ON				PORTB |= 0x01
#define LED_OFF				PORTB &= ~0x01

	uint16_t count_0, count_1 ;	//-impulse counters

//-JSON structure - mobile interface definition
const char json_counter[] PROGMEM ="{\"ty\":\"lout\",\"or\":\"V\",\"img\":\"3\",\"cs\":[{ \
\"ty\":\"TextView\",\"n\":\"c\",\"te\":\"00000\",\"tl\":\"bold\",\"ts\":\"60\",\"tc\":\"255,255,255\",\"w\":\"2\"}, \
{\"ty\":\"lout\",\"or\":\"H\",\"cs\":[ \
{\"ty\":\"Button\",\"n\":\"b1\",\"te\":\"RESET\",\"tc\":\"255,255,255\",\"bg\":\"20,30,127\",\"ts\":\"25\"}, \
{\"ty\":\"Button\",\"n\":\"b2\",\"tc\":\"255,255,255\",\"bg\":\"137,5,78\",\"ts\":\"25\"}]}, \
{\"ty\":\"TextView\",\"n\":\"s\",\"tc\":\"200,200,200\",\"ts\":\"30\",\"w\":\"3\"}]}" ;


//==========================================================================
//-MAIN
//==========================================================================
int main(void)
{
	uint8_t i, on_off ;
	uint16_t j ;

	DDRB = 0b00000001 ;
	PORTB = 0b00000011 ;
	DDRD = 0b00000010 ;
	PORTD = 0b00001111 ;

    //bbmobile_uart_init((uint16_t)(F_CPU/(9600*16l)-1)) ;   		//-9600
    bbmobile_uart_init(12) ;	//-UBRR reg with double UART speed -> 9600bps

	//-config external int0 and external int1
	EICRA = (1<<ISC01) | (1<<ISC11) ;	//-falling edge generates interrupt

	//-enable interrupts
	sei() ;

	//-wait for BBMobile modules answer
	do
	{
		if( bbmobile_send_2chip_ack("<hello\r\n") ==0 )	//-it takes about 500ms if timeout
			break ;
		LED_OFF ; bbmobile_wait_ms(100) ; LED_ON ;
	}while(1) ;

	//-set Bluetooth device name
	i = bbmobile_send_2chip_ack("<name,MOBILE COUNTER\r\n") ;

	//-set PIN
	//i = bbmobile_send_2chip_ack("<pin,123\r\n") ;
	//-delete PIN
	//i = bbmobile_send_2chip_ack("<pin,0\r\n") ;

	on_off =0 ;	//-counting is off now
	j =30000 ;	//-update counter screen data immedietely

	for(;;)		//-do it forever
	{
		//-wait for BLE connection
		while( bbmobile_check_ble_conn() ==0 )
		{
			LED_OFF ; bbmobile_wait_ms(900) ;
			LED_ON ; bbmobile_wait_ms(100) ;
		} ;

		//-send JSON structure to build mobile interface
		bbmobile_send_json(json_counter) ;
		
		//-display counters state and buttons description
		if(on_off)	//-if counting is on
			bbmobile_send_2app_ack("$set,s:te=\"COUNTING\",b2:te=\"STOP\"\r\n") ;
		else	//-if counting is off
			bbmobile_send_2app_ack("$set,s:te=\"STOPPED\",b2:te=\"GO\"\r\n") ;


		//-play with interface
		j =0 ;
		while(bbmobile_check_ble_conn())	//-until BLE connection is established
		{
			if( bbmobile_is_data() )		//-if there is new data from BBMobile
			{
				if(cmp_flash_ram_bytes("$by,b1", bbm_buf, 6))		//-if RESET button pressed
				{
					count_0 =0 ;
					count_1 =0 ;
					bbmobile_send_2app_ack("$tst,\"Counter RESET\"\r\n") ;
				}else if(cmp_flash_ram_bytes("$by,b2", bbm_buf, 6))	//-if GO/STOP button pressed
				{
					if(on_off)			//-counting is on now
					{
						on_off =0 ;		//-turn off counting
						EIMSK = 0 ; 	//-disable INT0 and INT1 external interuupts
						bbmobile_send_2app_ack("$set,s:te=\"STOPPED\",b2:te=\"GO\"\r\n") ;
						//bbmobile_send_2app_ack("$tst,\"COUNTER STOP\"\r\n") ;
					}else				//-counting is off now
					{
						on_off =1 ;		//-turn on counting
						EIFR = (1<<INT0) | (1<<INT1) ;	//-clear flags
						EIMSK = (1<<INT0) | (1<<INT1) ; //-enable INT0 and INT1 external interuupts
						bbmobile_send_2app_ack("$set,s:te=\"COUNTING\",b2:te=\"STOP\"\r\n") ;
						//bbmobile_send_2app_ack("$tst,\"COUNTER ON\"\r\n") ;
					}
				}
				//-enable rec new data from BBMobile
				bbmobile_rx_enable() ;
			}

			//-update data on mobile screen
			j++ ;
			if(j >= 30000)
			{
				j=0 ;
				bbmobile_send_2app("$set,c:te=\"") ;
				bbmobile_send_sigdec_2app(count_0,5) ;
				bbmobile_send_2app_ack("\"\r\n") ;
			}
		} ;

		//-BLE connection is closed here
		LED_OFF ;
	}

    return(0) ;
}

//==========================================================================
//-INT0 vector
//==========================================================================
SIGNAL(INT0_vect)
{
	count_0++ ;
}

//==========================================================================
//-INT1 vector
//==========================================================================
SIGNAL(INT1_vect)
{
	count_1++ ;
}

